% This Matlab code illustrates the proposed method LNWS on the inductance 
% plethysmography data set (IPD).
%
% 3 options for estimation can be used: 1) posterior mean 
%                                       2) posterior median
%                                       3) Bayes factor
%
% The hyperparameters are estimated through empirical Bayes method
% numerically maximizing the marginal distribution
% 
%  The program calls to:
%           marg_lnws.m -- Calculates the value of the marginal 
%                          log-likelihood for input hyperparameter values.
%           bayesrule_lnws.m -- Computes the posterior mean for the LNWS 
%                               method.
%           bayesrule_lnws_median.m -- Computes the posterior median for
%                                      the LNWS method.
%           bayesrule_lnws_bf.m -- Computes the Bayes factor for the LNWS
%                                  method.
%--------------------------------------------------------------------------
clear all;
close all;
set(0, 'DefaultAxesFontSize', 16);

load ipd.dat;  % IPD data set
N = length(ipd);
coarsest = floor(log2(log(N))) + 1;  % coarsest level of decomposition
finest_lev = log2(N)-1;  % finest level
filt = MakeONFilter('Symmlet', 8);  % wavelet filter
sn = ipd';

data_wd = FWT_PO (sn, coarsest-1, filt);  % wavelet transform performed one level after coarsest to get parents for detail coefficients
data_smowd = data_wd;
niz = data_wd(dyad(finest_lev));
sigma_hat = mad(niz,1)/0.6745;  % estimate noise

% Forming "thresholding statistics"
for i = coarsest:1:finest_lev
    ix=dyad(i);
    for j=ix(1:2:end)
        thetaj(round(j/2))=(data_wd(j)^2+data_wd(j+1)^2+data_wd(round(j/2))^2)/sigma_hat^2;
    end;
end;

% Empirical Bayes estimation of hyperparameters
% minimizing the negative of the marginal log-likelihood by 'interior-point' method
x0 = [repmat(0.5,1,finest_lev-coarsest+1) 0.001];
lb=[repmat(0,1,finest_lev-coarsest+1) 0];
ub=[repmat(1,1,finest_lev-coarsest+1) Inf];
options = optimset('Algorithm','interior-point','Display','off');
par_opt = fmincon(@(par) marg_lnws(par,thetaj,coarsest),x0,[],[],[],[],lb,ub,[],options);

% Bayesian shrinkage of energies
for i = (coarsest-1):1:(finest_lev-1),   
    thetajj=thetaj(dyad(i));
    th_tr = thetajj;
    nn = length(th_tr);   
    for  j = 1:nn,
        %th_tr(j) = bayesrule_lnws(thetajj(j), par_opt(i-(coarsest-2)), par_opt(end) ); % using posterior mean
        %th_tr(j) = bayesrule_lnws_median(thetajj(j), par_opt(i-(coarsest-2)), par_opt(end) ); % using posterior median
        th_tr(j) = bayesrule_lnws_bf(thetajj(j), par_opt(i-(coarsest-2)), par_opt(end) ); % using Bayes factor
    end;
    thetaj_smo(dyad(i))= th_tr;
    shrink(dyad(i))=sqrt(th_tr./thetajj);
end;

% Shrinkage of wavelet coefficients
shrink_factor(1:2:N-1)=shrink;
shrink_factor(2:2:N)=shrink;
shrink_factor(1:2^coarsest)=1; % preserve coarsest level  
data_smowd=shrink_factor.*data_wd;
                   
smo_data = IWT_PO (data_smowd, coarsest-1, filt); % inverse wavelet transfrom

subplot(2,1,1); plot((1:N)/N,sn,'LineWidth',1.0);
title('Inductance plethysmography data'); ylim([-0.2 1.2]);
subplot(2,1,2); plot((1:N)/N,smo_data,'r-','LineWidth',1.0);
title('LNWS reconstruction'); ylim([-0.2 1.2]);
